@@ -1,9 +1,14 @@
+from __future__ import absolute_import, division
+
 from twisted.internet import defer
 from twisted.python import log, failure
+from twisted.python.compat import unicode, intToBytes
 from twisted.web import server
 from twisted.web.iweb import IRenderable
 from twisted.web.resource import Resource, IResource, getChildForRequest
-from twisted.web.template import flattenString
+from twisted.web.server import NOT_DONE_YET
+from twisted.web.template import renderElement
+
 from werkzeug.exceptions import HTTPException
 
 from klein.interfaces import IKleinRequest
@@ -73,20 +78,21 @@ def _extractURLparts(request):
     server_port = request.getHost().port
     if (bool(request.isSecure()), server_port) not in [
             (True, 443), (False, 80)]:
-        server_name = '%s:%d' % (server_name, server_port)
-    script_name = ''
+        server_name = server_name + b":" + intToBytes(server_port)
+
+    script_name = b''
     if request.prepath:
-        script_name = '/'.join(request.prepath)
+        script_name = b'/'.join(request.prepath)
 
-        if not script_name.startswith('/'):
-            script_name = '/' + script_name
+        if not script_name.startswith(b'/'):
+            script_name = b'/' + script_name
 
-    path_info = ''
+    path_info = b''
     if request.postpath:
-        path_info = '/'.join(request.postpath)
+        path_info = b'/'.join(request.postpath)
 
-        if not path_info.startswith('/'):
-            path_info = '/' + path_info
+        if not path_info.startswith(b'/'):
+            path_info = b'/' + path_info
 
     url_scheme = u'https' if request.isSecure() else u'http'
 
@@ -199,19 +205,25 @@ class KleinResource(Resource):
                 if isinstance(r, unicode):
                     r = r.encode('utf-8')
 
-                if r is not None:
+                if (r is not None) and (r != NOT_DONE_YET):
                     request.write(r)
 
                 if not request_finished[0]:
                     request.finish()
 
         def process(r):
+            """
+            Recursively go through r and any child Resources until something
+            returns an IRenderable, then render it and let the result of that
+            bubble back up.
+            """
+
             if IResource.providedBy(r):
                 request.render(getChildForRequest(r, request))
                 return _StandInResource
 
             if IRenderable.providedBy(r):
-                return flattenString(request, r).addCallback(process)
+                return renderElement(request, r)
 
             return r
 
